
//Collector Sample Code using Pushbullet
// Kevin Darrah

// Includes - may not all be needed
#include <FS.h>                 
#include <Arduino.h>
#include <Wire.h>
#include <ACROBOTIC_SSD1306.h>    //https://github.com/acrobotic/Ai_Ardulib_SSD1306

#include <ESP8266WiFi.h>          //https://github.com/esp8266/Arduino
#include <DNSServer.h>
#include <WiFiManager.h>          //https://github.com/tzapu/WiFiManager
#include <WiFiClientSecure.h>
#include <ArduinoJson.h>          //https://github.com/bblanchon/ArduinoJson
#include <ESP8266mDNS.h>
#include <WiFiUdp.h>
#include <ArduinoOTA.h>
#include <WebSocketsClient.h>     //https://github.com/Links2004/arduinoWebSockets
#include <Hash.h>
#include <time.h>

WebSocketsClient webSocket;

// PINS!!!
#define LEDpin 0
#define buzzerPin 14
#define noisePin 12
#define motionPin 16
#define buttonPin 2
#define USE_SERIAL Serial//serial for debugging

WiFiManager wifiManager;

//FUNCTIONS
void PushStatus(const char *MessageBody, const char *MessageTitle, const char *MessageType, ...); // to push messages
void saveConfigCallback ();
void readFile();
void setupWiFi();
boolean WiFiConnection();
void checkTrigger();
void startOTA();
void longPress();
void getPush();
void webSocketEvent(WStype_t type, uint8_t * payload, size_t length);
float getTemperature();
void checkSensors();

//globals for credentials
char tokenKey[40];
char pusherName[40];
char triggerMessage[100];
boolean resetFlag = false;
int numberOfDisconnects = 0;


//flag for saving data
bool shouldSaveConfig = false;

//other globals for collector
unsigned long msCounter;
int noiseTimeOut = 0;
bool sensorTriggered = false;
bool silentSensor = false;
bool firstBoot = true;

void setup() {
  Wire.begin();//for OLED
  oled.init();                      // Initialze SSD1306 OLED display
  oled.setFont(font8x8);            // Set font type (default 8x8)

  oled.setTextXY(0, 0);
  oled.putString("Booting Up...");
  for (int i = 0; i < 500; i++)
    oled.putString(" ");

  pinMode(LEDpin, OUTPUT);
  digitalWrite(LEDpin, LOW);
  pinMode(noisePin, INPUT);
  pinMode(motionPin, INPUT);
  pinMode(buzzerPin, OUTPUT);
  
  USE_SERIAL.begin(115200);
  USE_SERIAL.setDebugOutput(true);
  USE_SERIAL.println();
  USE_SERIAL.println();
  USE_SERIAL.println();

  for (uint8_t t = 4; t > 0; t--) {
    //USE_SERIAL.printf("[SETUP] BOOT WAIT %d...\n", t);
    USE_SERIAL.flush();
    delay(1000);
  }
  readFile();
  //setupWiFi();
  WiFiConnection();
  configTime(-4 * 3600, 0, "pool.ntp.org", "time.nist.gov");

  char streamURL[100];
  sprintf(streamURL, "wss://stream.pushbullet.com/websocket/%s", tokenKey);

  webSocket.beginSSL("stream.pushbullet.com", 443, streamURL);
  webSocket.onEvent(webSocketEvent);
  startOTA();
  oled.setTextXY(1, 0);
  oled.putString("Ready!!");

  while (!time(nullptr)) {
    Serial.print(".");
    delay(1000);
  }
  getPush();
}

void loop() {

  if (digitalRead(buttonPin) == LOW) {//just to silence the microphone and motion sensor triggers
    silentSensor = !silentSensor;
    oled.setTextXY(7, 8);
    if (silentSensor)
      oled.putString("SILENT");
    else
      oled.putString("      ");
    delay(500);
  }
  
  if (msCounter > 1000) {//1sec ticker
    noiseTimeOut++;
    if (noiseTimeOut >= 300)
      noiseTimeOut = 300;
    msCounter = 0;
    checkSensors();
    float ambientTemp = (((analogRead(A0) * 3.3 / 1023) - 0.75) / 0.010) + 25;
    char tempBuf[5] = "";
    dtostrf(ambientTemp, 3, 1, tempBuf);

    char tempDisplay[20];
    sprintf(tempDisplay, "%sF  ", tempBuf);

    oled.setTextXY(7, 0);
    oled.putString(tempDisplay);
  }

  
  ArduinoOTA.handle();//check for OTA
  webSocket.loop();
  if ( WiFi.status()  != WL_CONNECTED) {
    Serial.println("lost WiFi Connection, attempting to reconnect");
    WiFiConnection();
  }
  //todo add a button to reset Wifi settings
  delay(1);
  msCounter++;
}






